@@ -7,35 +7,37 @@
 import json
 import os
 import os.path as osp
-from os.path import join as pjoin
 import sys
+from os.path import join as pjoin
 
-from jupyter_core.application import JupyterApp, base_aliases, base_flags
-from jupyter_core.application import NoStart
-from jupyterlab_server import slugify, WORKSPACE_EXTENSION
+from jupyter_core.application import JupyterApp, NoStart, base_aliases, base_flags
+from jupyter_server._version import version_info as jpserver_version_info
 from jupyter_server.serverapp import flags
 from jupyter_server.utils import url_path_join as ujoin
-from jupyter_server._version import version_info as jpserver_version_info
+
+from jupyterlab_server import WORKSPACE_EXTENSION, LabServerApp, slugify
+from nbclassic.shim import NBClassicConfigShimMixin
 from traitlets import Bool, Instance, Unicode, default
 
-from nbclassic.shim import NBClassicConfigShimMixin
-from jupyterlab_server import LabServerApp
-
 from ._version import __version__
-from .debuglog import DebugLogFileMixin
 from .commands import (
-    DEV_DIR, HERE,
-    build, clean, get_app_dir, get_app_version, get_user_settings_dir,
-    get_workspaces_dir, AppOptions, pjoin,
-    ensure_core, ensure_dev, watch, watch_dev, ensure_app
+    DEV_DIR, HERE, AppOptions, build, clean, ensure_app,
+    ensure_core, ensure_dev, get_app_dir, get_app_version,
+    get_user_settings_dir, get_workspaces_dir, pjoin, watch,
+    watch_dev
 )
 from .coreconfig import CoreConfig
-from .handlers.build_handler import build_path, Builder, BuildHandler
-from .handlers.extension_manager_handler import (
-    extensions_handler_path, ExtensionManager, ExtensionHandler
-)
+from .debuglog import DebugLogFileMixin
+from .handlers.build_handler import Builder, BuildHandler, build_path
 from .handlers.error_handler import ErrorHandler
+from .handlers.extension_manager_handler import ExtensionHandler, ExtensionManager, extensions_handler_path
+from .handlers.yjs_echo_ws import YjsEchoWebSocket
 
+# TODO: remove when oldest compatible jupyterlab_server contains license tooling
+try:
+    from jupyterlab_server import LicensesApp
+except ImportError:
+    LicensesApp = None
 
 DEV_NOTE = """You're running JupyterLab from source.
 If you're working on the TypeScript sources of JupyterLab, try running
@@ -70,6 +72,11 @@ build_flags['no-minimize'] = (
     {'LabBuildApp': {'minimize': False}},
     "Do not minimize a production build."
 )
+build_flags['splice-source'] = (
+    {'LabBuildApp': {'splice_source': True}},
+    "Splice source packages into app directory."
+)
+
 
 version = __version__
 app_version = get_app_version()
@@ -146,10 +153,13 @@ class LabBuildApp(JupyterApp, DebugLogFileMixin):
     pre_clean = Bool(False, config=True,
         help="Whether to clean before building (defaults to False)")
 
+    splice_source = Bool(False, config=True,
+        help="Splice source packages into app directory.")
+
     def start(self):
         app_dir = self.app_dir or get_app_dir()
         app_options = AppOptions(
-            app_dir=app_dir, logger=self.log, core_config=self.core_config
+            app_dir=app_dir, logger=self.log, core_config=self.core_config, splice_source=self.splice_source
         )
         self.log.info('JupyterLab %s', version)
         with self.debug_logging():
@@ -411,6 +421,46 @@ class LabWorkspaceApp(JupyterApp):
         self.exit(0)
 
 
+if LicensesApp is not None:
+    class LabLicensesApp(LicensesApp):
+        version = version
+
+        dev_mode = Bool(
+            False,
+            config=True,
+            help="""Whether to start the app in dev mode. Uses the unpublished local
+            JavaScript packages in the `dev_mode` folder.  In this case JupyterLab will
+            show a red stripe at the top of the page.  It can only be used if JupyterLab
+            is installed as `pip install -e .`.
+            """,
+        )
+
+        app_dir = Unicode(
+            "", config=True, help="The app directory for which to show licenses"
+        )
+
+        aliases = {
+            **LicensesApp.aliases,
+            "app-dir": "LabLicensesApp.app_dir",
+        }
+
+        flags = {
+            **LicensesApp.flags,
+            "dev-mode": (
+                {"LabLicensesApp": {"dev_mode": True}},
+                "Start the app in dev mode for running from source.",
+            ),
+        }
+
+        @default('app_dir')
+        def _default_app_dir(self):
+            return get_app_dir()
+
+        @default('static_dir')
+        def _default_static_dir(self):
+            return pjoin(self.app_dir, 'static')
+
+
 aliases = dict(base_aliases)
 aliases.update({
     'ip': 'ServerApp.ip',
@@ -482,6 +532,10 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         {'LabApp': {'watch': True}},
         "Start the app in watch mode."
     )
+    flags['splice-source'] = (
+        {'LabApp': {'splice_source': True}},
+        "Splice source packages into app directory."
+    )
     flags['expose-app-in-browser'] = (
         {'LabApp': {'expose_app_in_browser': True}},
         "Expose the global app instance to browser via window.jupyterlab."
@@ -490,6 +544,10 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         {'LabApp': {'extensions_in_dev_mode': True}},
         "Load prebuilt extensions in dev-mode."
     )
+    flags['collaborative'] = (
+        {'LabApp': {'collaborative': True}},
+        "Whether to enable collaborative mode."
+    )
 
     subcommands = dict(
         build=(LabBuildApp, LabBuildApp.description.splitlines()[0]),
@@ -500,6 +558,12 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         workspaces=(LabWorkspaceApp, LabWorkspaceApp.description.splitlines()[0])
     )
 
+    # TODO: remove when oldest compatible jupyterlab_server contains license tooling
+    if LicensesApp is not None:
+        subcommands.update(
+            licenses=(LabLicensesApp, LabLicensesApp.description.splitlines()[0])
+        )
+
     default_url = Unicode('/lab', config=True,
         help="The default URL to redirect to from `/`")
 
@@ -541,9 +605,15 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
     watch = Bool(False, config=True,
         help="Whether to serve the app in watch mode")
 
+    splice_source = Bool(False, config=True,
+        help="Splice source packages into app directory.")
+
     expose_app_in_browser = Bool(False, config=True,
         help="Whether to expose the global app instance to browser via window.jupyterlab")
 
+    collaborative = Bool(False, config=True,
+        help="Whether to enable collaborative mode.")
+
     @default('app_dir')
     def _default_app_dir(self):
         app_dir = get_app_dir()
@@ -651,6 +721,7 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         page_config['token'] = self.serverapp.token
         page_config['exposeAppInBrowser'] = self.expose_app_in_browser
         page_config['quitButton'] = self.serverapp.quit_button
+        page_config['collaborative'] = self.collaborative
 
         # Client-side code assumes notebookVersion is a JSON-encoded string
         page_config['notebookVersion'] = json.dumps(jpserver_version_info)
@@ -658,11 +729,15 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         self.log.info('JupyterLab extension loaded from %s' % HERE)
         self.log.info('JupyterLab application directory is %s' % self.app_dir)
 
-        build_handler_options = AppOptions(logger=self.log, app_dir=self.app_dir, labextensions_path = self.extra_labextensions_path + self.labextensions_path)
+        build_handler_options = AppOptions(logger=self.log, app_dir=self.app_dir, labextensions_path = self.extra_labextensions_path + self.labextensions_path, splice_source=self.splice_source)
         builder = Builder(self.core_mode, app_options=build_handler_options)
         build_handler = (build_path, BuildHandler, {'builder': builder})
         handlers.append(build_handler)
 
+        # Yjs Echo WebSocket handler
+        yjs_echo_handler = (r"/api/yjs/(.*)", YjsEchoWebSocket)
+        handlers.append(yjs_echo_handler)
+
         errored = False
 
         if self.core_mode:
@@ -673,6 +748,8 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
                 ensure_dev(self.log)
                 self.log.info(DEV_NOTE)
         else:
+            if self.splice_source:
+                ensure_dev(self.log)
             msgs = ensure_app(self.app_dir)
             if msgs:
                 [self.log.error(msg) for msg in msgs]
