@@ -3,30 +3,25 @@
 
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
-from glob import glob
-import json
-import os
-import shutil
-import sys
-import traceback
 
+import os
+import sys
 from copy import copy
 
-from jupyter_core.application import JupyterApp, base_flags, base_aliases
-from jupyter_core.paths import jupyter_path
-from jupyterlab.coreconfig import CoreConfig
-from jupyterlab.debuglog import DebugLogFileMixin
+from jupyter_core.application import JupyterApp, base_aliases, base_flags
 from traitlets import Bool, Instance, List, Unicode, default
 
-from .commands import (
-    install_extension, uninstall_extension, list_extensions,
-    enable_extension, disable_extension, check_extension,
-    link_package, unlink_package, build, get_app_version, HERE,
-    update_extension, AppOptions,
-)
-from .federated_labextensions import develop_labextension_py, build_labextension, watch_labextension
-from .labapp import LabApp
+from jupyterlab.coreconfig import CoreConfig
+from jupyterlab.debuglog import DebugLogFileMixin
 
+from .commands import (
+    HERE, AppOptions, build, check_extension,
+    disable_extension, enable_extension, get_app_version,
+    install_extension, link_package, list_extensions,
+    uninstall_extension, unlink_package, update_extension
+)
+from .federated_labextensions import build_labextension, develop_labextension_py, watch_labextension
+from .labapp import LabApp
 
 flags = dict(base_flags)
 flags['no-build'] = (
@@ -45,6 +40,10 @@ flags['clean'] = (
     {'BaseExtensionApp': {'should_clean': True}},
     "Cleanup intermediate files after the action."
 )
+flags['splice-source'] = (
+    {'BaseExtensionApp': {'splice_source': True}},
+    "Splice source packages into app directory."
+)
 
 check_flags = copy(flags)
 check_flags['installed'] = (
@@ -113,6 +112,9 @@ class BaseExtensionApp(JupyterApp, DebugLogFileMixin):
     should_clean = Bool(False, config=True,
         help="Whether temporary files should be cleaned up after building jupyterlab")
 
+    splice_source = Bool(False, config=True,
+        help="Splice source packages into app directory.")
+
     labextensions_path = List(Unicode(), help='The standard paths to look in for prebuilt JupyterLab extensions')
 
     @default('labextensions_path')
@@ -121,6 +123,11 @@ class BaseExtensionApp(JupyterApp, DebugLogFileMixin):
         lab.load_config_file()
         return lab.extra_labextensions_path + lab.labextensions_path
 
+    @default('splice_source')
+    def _default_splice_source(self):
+        version = get_app_version(AppOptions(app_dir=self.app_dir))
+        return version.endswith('-spliced')
+
     def start(self):
         if self.app_dir and self.app_dir.startswith(HERE):
             raise ValueError('Cannot run lab extension commands in core app')
@@ -129,7 +136,7 @@ class BaseExtensionApp(JupyterApp, DebugLogFileMixin):
             if ans and self.should_build:
                 production = None if self.dev_build is None else not self.dev_build
                 app_options = AppOptions(app_dir=self.app_dir, logger=self.log,
-                      core_config=self.core_config)
+                      core_config=self.core_config, splice_source=self.splice_source)
                 build(clean_staging=self.should_clean,
                       production = production, minimize = self.minimize, app_options=app_options)
 
@@ -150,8 +157,8 @@ class InstallLabExtensionApp(BaseExtensionApp):
 
     This installs JupyterLab extensions similar to yarn add or npm install.
 
-    Pass a list of comma seperate names to the --pin-version-as flag
-    to use as alises for the packages providers. This is useful to
+    Pass a list of comma separate names to the --pin-version-as flag
+    to use as aliases for the packages providers. This is useful to
     install multiple versions of the same extension.
     These can be uninstalled with the alias you provided
     to the flag, similar to the "alias" feature of yarn add.
@@ -181,7 +188,7 @@ class InstallLabExtensionApp(BaseExtensionApp):
 
 
 class DevelopLabExtensionApp(BaseExtensionApp):
-    desciption = "Develop labextension"
+    description = "Develop labextension"
     flags = develop_flags
 
     user = Bool(False, config=True, help="Whether to do a user install")
@@ -210,7 +217,7 @@ class BuildLabExtensionApp(BaseExtensionApp):
         help="Build in development mode")
 
     source_map = Bool(False, config=True,
-        help="Generage source maps")
+        help="Generate source maps")
 
     core_path = Unicode(os.path.join(HERE, 'staging'), config=True,
         help="Directory containing core application package.json file")
@@ -235,7 +242,7 @@ class WatchLabExtensionApp(BaseExtensionApp):
         help="Build in development mode")
 
     source_map = Bool(False, config=True,
-        help="Generage source maps")
+        help="Generate source maps")
 
     core_path = Unicode(os.path.join(HERE, 'staging'), config=True,
         help="Directory containing core application package.json file")
